@@ -42,10 +42,12 @@
 #include "fault-injection.h"
 #include "memfd.h"
 #include "hugetlb.h"
+#include "pidfd.h"
 
 #include "protobuf.h"
 #include "images/fdinfo.pb-c.h"
 #include "images/mnt.pb-c.h"
+#include "pidfd.pb-c.h"
 #include "plugin.h"
 
 #include <stdlib.h>
@@ -118,7 +120,8 @@ bool handle_vma_plugin(int *fd, struct stat *stat)
 	return true;
 }
 
-static void __parse_vmflags(char *buf, u32 *flags, u64 *madv, int *io_pf)
+static void __parse_vmflags(char *buf, u32 *flags, u64 *madv, int *io_pf,
+			    int *shstk)
 {
 	char *tok;
 
@@ -162,6 +165,9 @@ static void __parse_vmflags(char *buf, u32 *flags, u64 *madv, int *io_pf)
 		if (_vmflag_match(tok, "io") || _vmflag_match(tok, "pf"))
 			*io_pf = 1;
 
+		if (_vmflag_match(tok, "ss"))
+			*shstk = 1;
+
 		/*
 		 * Anything else is just ignored.
 		 */
@@ -172,14 +178,21 @@ static void __parse_vmflags(char *buf, u32 *flags, u64 *madv, int *io_pf)
 
 void parse_vmflags(char *buf, u32 *flags, u64 *madv, int *io_pf)
 {
-	__parse_vmflags(buf, flags, madv, io_pf);
+	int shstk = 0;
+
+	__parse_vmflags(buf, flags, madv, io_pf, &shstk);
 }
 
 static void parse_vma_vmflags(char *buf, struct vma_area *vma_area)
 {
 	int io_pf = 0;
+	int shstk = 0;
 
-	__parse_vmflags(buf, &vma_area->e->flags, &vma_area->e->madv, &io_pf);
+	__parse_vmflags(buf, &vma_area->e->flags, &vma_area->e->madv, &io_pf,
+			&shstk);
+
+	if (shstk)
+		vma_area->e->status |= VMA_AREA_SHSTK;
 
 	/*
 	 * vmsplice doesn't work for VM_IO and VM_PFNMAP mappings, the
@@ -566,7 +579,8 @@ static int handle_vma(pid_t pid, struct vma_area *vma_area, const char *file_pat
 	} else if (!strcmp(file_path, "[vdso]")) {
 		if (handle_vdso_vma(vma_area))
 			goto err;
-	} else if (!strcmp(file_path, "[vvar]")) {
+	} else if (!strcmp(file_path, "[vvar]") ||
+		   !strcmp(file_path, "[vvar_vclock]")) {
 		if (handle_vvar_vma(vma_area))
 			goto err;
 	} else if (!strcmp(file_path, "[heap]")) {
@@ -758,7 +772,7 @@ static int task_size_check(pid_t pid, VmaEntry *entry)
 
 int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list, dump_filemap_t dump_filemap)
 {
-	struct vma_area *vma_area = NULL;
+	struct vma_area *vma_area = NULL, *prev_vma_area = NULL;
 	unsigned long start, end, pgoff, prev_end = 0;
 	char r, w, x, s;
 	int ret = -1, vm_file_fd = -1;
@@ -800,8 +814,22 @@ int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list, dump_filemap_t du
 				continue;
 		}
 
-		if (vma_area && vma_list_add(vma_area, vma_area_list, &prev_end, &vfi, &prev_vfi))
-			goto err;
+		if (vma_area && vma_area_is(vma_area, VMA_AREA_VVAR) &&
+		    prev_vma_area && vma_area_is(prev_vma_area, VMA_AREA_VVAR)) {
+			if (prev_vma_area->e->end != vma_area->e->start) {
+				pr_err("two nonconsecutive vvar vma-s: "
+				       "%" PRIx64 "-%" PRIx64 " %" PRIx64 "-%" PRIx64 "\n",
+				       prev_vma_area->e->start, prev_vma_area->e->end,
+				       vma_area->e->start, vma_area->e->end);
+				goto err;
+			}
+			/* Merge all vvar vma-s into one. */
+			prev_vma_area->e->end = vma_area->e->end;
+		} else {
+			if (vma_area && vma_list_add(vma_area, vma_area_list, &prev_end, &vfi, &prev_vfi))
+				goto err;
+			prev_vma_area = vma_area;
+		}
 
 		if (eof)
 			break;
@@ -1043,7 +1071,7 @@ int parse_pid_status(pid_t pid, struct seize_task_status *ss, void *data)
 	if (bfdopenr(&f))
 		return -1;
 
-	while (done < 13) {
+	while (done < 14) {
 		str = breadline(&f);
 		if (str == NULL)
 			break;
@@ -1127,6 +1155,13 @@ int parse_pid_status(pid_t pid, struct seize_task_status *ss, void *data)
 			continue;
 		}
 
+		if (!strncmp(str, "CapAmb:", 7)) {
+			if (cap_parse(str + 8, cr->cap_amb))
+				goto err_parse;
+			done++;
+			continue;
+		}
+
 #ifdef ENABLE_SECCOMP
 		if (!strncmp(str, "Seccomp:", 8)) {
 			if (sscanf(str + 9, "%d", &cr->s.seccomp_mode) != 1) {
@@ -1172,7 +1207,7 @@ int parse_pid_status(pid_t pid, struct seize_task_status *ss, void *data)
 	}
 
 	/* seccomp and nspids are optional */
-	expected_done = (parsed_seccomp ? 12 : 11);
+	expected_done = (parsed_seccomp ? 13 : 12);
 	if (kdat.has_nspid)
 		expected_done++;
 	if (done == expected_done)
@@ -2156,6 +2191,33 @@ static int parse_fdinfo_pid_s(int pid, int fd, int type, void *arg)
 			if (ret)
 				goto parse_err;
 
+			entry_met = true;
+			continue;
+		}
+		if (fdinfo_field(str, "ino") || fdinfo_field(str, "NSpid") || fdinfo_field(str, "Pid")) {
+			struct pidfd_dump_info *pidfd_info = arg;
+
+			if (type != FD_TYPES__PIDFD)
+				continue;
+
+			if (fdinfo_field(str, "ino")) {
+				ret = sscanf(str, "%*s %u", &pidfd_info->pidfe.ino);
+				if (ret != 1)
+					goto parse_err;
+			} else if (fdinfo_field(str, "Pid")) {
+				ret = sscanf(str, "%*s %d", &pidfd_info->pid);
+				if (ret != 1)
+					goto parse_err;
+			} else if (fdinfo_field(str, "NSpid")) {
+				char *last;
+
+				last = strrchr(str, '\t');
+				if (!last || sscanf(last, "%d", &pidfd_info->pidfe.nspid) != 1) {
+					pr_err("Unable to parse: %s\n", str);
+					goto parse_err;
+				}
+			}
+
 			entry_met = true;
 			continue;
 		}
